@@ -1,24 +1,7 @@
-# The MIT License (MIT)
+# SPDX-FileCopyrightText: 2019 ladyada for Adafruit Industries
 #
-# Copyright (c) 2019 ladyada for Adafruit Industries
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# SPDX-License-Identifier: MIT
+
 """
 `adafruit_mlx90640`
 ================================================================================
@@ -96,8 +79,8 @@ class MLX90640:  # pylint: disable=too-many-instance-attributes
     cpAlpha = [0] * 2
     cpOffset = [0] * 2
     ilChessC = [0] * 3
-    brokenPixels = [0xFFFF] * 5
-    outlierPixels = [0xFFFF] * 5
+    brokenPixels = []
+    outlierPixels = []
     cpKta = 0
     cpKv = 0
 
@@ -116,7 +99,7 @@ class MLX90640:  # pylint: disable=too-many-instance-attributes
 
     @property
     def refresh_rate(self):
-        """ How fast the MLX90640 will spit out data. Start at lowest speed in
+        """How fast the MLX90640 will spit out data. Start at lowest speed in
         RefreshRate and then slowly increase I2C clock rate and rate until you
         max out. The sensor does not like it if the I2C host cannot 'keep up'!"""
         controlRegister = [0]
@@ -132,9 +115,9 @@ class MLX90640:  # pylint: disable=too-many-instance-attributes
         self._I2CWriteWord(0x800D, value)
 
     def getFrame(self, framebuf):
-        """ Request both 'halves' of a frame from the sensor, merge them
+        """Request both 'halves' of a frame from the sensor, merge them
         and calculate the temperature in C for each of 32x24 pixels. Placed
-        into the 768-element array passed in! """
+        into the 768-element array passed in!"""
         emissivity = 0.95
         tr = 23.15
         mlx90640Frame = [0] * 834
@@ -266,6 +249,11 @@ class MLX90640:  # pylint: disable=too-many-instance-attributes
             )
 
         for pixelNumber in range(768):
+            if self._IsPixelBad(pixelNumber):
+                # print("Fixing broken pixel %d" % pixelNumber)
+                result[pixelNumber] = -273.15
+                continue
+
             ilPattern = pixelNumber // 32 - (pixelNumber // 64) * 2
             chessPattern = ilPattern ^ (pixelNumber - (pixelNumber // 2) * 2)
             conversionPattern = (
@@ -744,30 +732,66 @@ class MLX90640:  # pylint: disable=too-many-instance-attributes
         self.ilChessC = ilChessC
 
     def _ExtractDeviatingPixels(self):
-        self.brokenPixels = [0xFFFF] * 5
-        self.outlierPixels = [0xFFFF] * 5
-
+        # pylint: disable=too-many-branches
         pixCnt = 0
-        brokenPixCnt = 0
-        outlierPixCnt = 0
 
-        while (pixCnt < 768) and (brokenPixCnt < 5) and (outlierPixCnt < 5):
+        while (
+            (pixCnt < 768)
+            and (len(self.brokenPixels) < 5)
+            and (len(self.outlierPixels) < 5)
+        ):
             if eeData[pixCnt + 64] == 0:
-                self.brokenPixels[brokenPixCnt] = pixCnt
-                brokenPixCnt += 1
+                self.brokenPixels.append(pixCnt)
             elif (eeData[pixCnt + 64] & 0x0001) != 0:
-                self.outlierPixels[outlierPixCnt] = pixCnt
-                outlierPixCnt += 1
+                self.outlierPixels.append(pixCnt)
             pixCnt += 1
 
-        if brokenPixCnt > 4:
+        if len(self.brokenPixels) > 4:
             raise RuntimeError("More than 4 broken pixels")
-        if outlierPixCnt > 4:
+        if len(self.outlierPixels) > 4:
             raise RuntimeError("More than 4 outlier pixels")
-        if (brokenPixCnt + outlierPixCnt) > 4:
+        if (len(self.brokenPixels) + len(self.outlierPixels)) > 4:
             raise RuntimeError("More than 4 faulty pixels")
-        # print("Found %d broken pixels, %d outliers" % (brokenPixCnt, outlierPixCnt))
-        # TODO INCOMPLETE
+        # print("Found %d broken pixels, %d outliers"
+        #         % (len(self.brokenPixels), len(self.outlierPixels)))
+
+        for brokenPixel1, brokenPixel2 in self._UniqueListPairs(self.brokenPixels):
+            if self._ArePixelsAdjacent(brokenPixel1, brokenPixel2):
+                raise RuntimeError("Adjacent broken pixels")
+
+        for outlierPixel1, outlierPixel2 in self._UniqueListPairs(self.outlierPixels):
+            if self._ArePixelsAdjacent(outlierPixel1, outlierPixel2):
+                raise RuntimeError("Adjacent outlier pixels")
+
+        for brokenPixel in self.brokenPixels:
+            for outlierPixel in self.outlierPixels:
+                if self._ArePixelsAdjacent(brokenPixel, outlierPixel):
+                    raise RuntimeError("Adjacent broken and outlier pixels")
+
+    def _UniqueListPairs(self, inputList):
+        # pylint: disable=no-self-use
+        for i, listValue1 in enumerate(inputList):
+            for listValue2 in inputList[i + 1 :]:
+                yield (listValue1, listValue2)
+
+    def _ArePixelsAdjacent(self, pix1, pix2):
+        # pylint: disable=no-self-use
+        pixPosDif = pix1 - pix2
+
+        if -34 < pixPosDif < -30:
+            return True
+        if -2 < pixPosDif < 2:
+            return True
+        if 30 < pixPosDif < 34:
+            return True
+
+        return False
+
+    def _IsPixelBad(self, pixel):
+        if pixel in self.brokenPixels or pixel in self.outlierPixels:
+            return True
+
+        return False
 
     def _I2CWriteWord(self, writeAddress, data):
         cmd = bytearray(4)
